from psychopy import visual, core, event
import random

# Setup
monitor_width = 1024  # Replace with actual width
monitor_height = 768  # Replace with actual height
win = visual.Window([monitor_width, monitor_height], units="pix")
mouse = event.Mouse(win=win)  # Persistent mouse object

# Variables
CIRCLE_DURATION = 3.0
MAX_RADIUS = 30
circles = []
next_circle_time = 0
globalClock = core.Clock()

# Data Logging Structures
clicks = []  # To store click times
circles_info = []  # To store info about each circle
all_click_times = []  # Time from start for every mouse click
circle_click_times = []  # Time from start for every circle click

# Display the stats box
DISPLAY_STATS = True  # Set this based on your needs
total_clicks = 0
circles_clicked = 0
total_circles = 0


# Circle Creation Function
def createCircle():
    # Adjusted position calculation here
    posX = random.uniform(-monitor_width/2 + MAX_RADIUS, monitor_width/2 - MAX_RADIUS)
    posY = random.uniform(-monitor_height/2 + MAX_RADIUS, monitor_height/2 - MAX_RADIUS)
    return {
        "shape": visual.Circle(win, radius=0, pos=[posX, posY], fillColor='red', lineColor='black', lineWidth=1),
        "createTime": globalClock.getTime(),
        "clicked": False
    }


def calculate_radius(time_elapsed, total_duration, max_radius):
    """
    Calculate the radius of a circle based on the elapsed time.
    
    Parameters:
    - time_elapsed: The time elapsed since the circle was created.
    - total_duration: The total duration for the circle's animation.
    - max_radius: The maximum radius the circle should reach.

    Returns:
    - The calculated radius for the current frame.
    """
    if time_elapsed < total_duration / 2:
        # Grow the circle
        return max_radius * (2 * time_elapsed / total_duration)
    else:
        # Shrink the circle
        return max_radius * (2 * (1 - time_elapsed / total_duration))


# Create a text stimulus for displaying statistics
stats_text = visual.TextStim(win,
                             pos=(monitor_width * 0.5 - 10, monitor_height * 0.5 - 10),
                             alignText='right',
                             anchorHoriz='right',
                             anchorVert='top',
                             text="Initializing...",
                             color='white')

was_mouse_pressed = False  # Track if the mouse was pressed in the previous frame

# Main Loop
while globalClock.getTime() < 12:
    # Create Circles at Intervals
    if globalClock.getTime() >= next_circle_time:
        circles.append(createCircle())
        total_circles += 1  # Increment the number of circles created
        next_circle_time += 0.4


    # Check for mouse clicks with debouncing
    is_mouse_pressed = mouse.getPressed()[0]  # Current frame's mouse state
    
    # Logic for when a new click is detected 
    if is_mouse_pressed and not was_mouse_pressed:  # Detect change from not pressed to pressed
        total_clicks += 1  # Increment total clicks only on the change
        new_click_detected = True  # Flag that a new click has been detected
    else:
        new_click_detected = False  # Reset the flag if no new click is detected


    was_mouse_pressed = is_mouse_pressed  # Update the tracking variable for the next frame


    # Update Circles
    for circle in list(circles):
        time_elapsed = globalClock.getTime() - circle["createTime"]
        all_click_times.append(time_elapsed)  # Log every mouse click
        if not circle["clicked"] and time_elapsed < CIRCLE_DURATION:
            # Update radius
            new_radius = calculate_radius(time_elapsed, CIRCLE_DURATION, MAX_RADIUS)
            circle["shape"].setRadius(new_radius)
            circle["shape"].draw()
            # Click Detection
            if new_click_detected and circle["shape"].contains(mouse.getPos()):
                circles_clicked += 1  # Increment circles clicked here
                circle_click_time = globalClock.getTime()
                circle_click_times.append(circle_click_time)  # Log every circle click
                clicks.append(circle_click_time)
                circle["clicked"] = True
                circle["clickTime"] = circle_click_time - circle["createTime"]
                circles_info.append({
                    "appearedTime": circle["createTime"],
                    "clickedTime": circle_click_time,
                    "clickedBeforeDisappear": circle_click_time < circle["createTime"] + CIRCLE_DURATION
                })
                circle["shape"].fillColor = 'green'  # Immediate visual feedback

    # Inside the loop, update the statistics
    if DISPLAY_STATS:
        elapsed_time = globalClock.getTime()
        stats_text.setText(f"Elapsed Time: {elapsed_time:.2f}s\nTotal Clicks: {total_clicks}\nCircles Clicked: {circles_clicked}\nCircles Created: {total_circles}")
        stats_text.draw()

    # Refresh screen and remove completed circles
    # Flip Window
    win.flip()
    # Remove completed circles
    circles = [circle for circle in circles if not circle["clicked"] and globalClock.getTime() - circle["createTime"] < CIRCLE_DURATION]

# Calculate Summary Statistics
circles_clicked = sum(1 for c in circles_info if c["clickedBeforeDisappear"])
average_targets_per_second = total_circles / 66
average_reaction_time = sum(c["clickedTime"] for c in circles_info if c["clickedBeforeDisappear"]) / circles_clicked if circles_clicked > 0 else None

# Log Summary Statistics
print("Total Clicks on Screen: ", total_clicks)
print("Total Circles Clicked: ", circles_clicked)
print("Number of Circles that were clicked vs how many appeared: {}/{}".format(circles_clicked, total_circles))
print("Average Number of Targets per Second: ", average_targets_per_second)
print("Average Reaction Time to Click a Target (ms): ", average_reaction_time * 1000 if average_reaction_time else "N/A")

# Cleanup
win.close()
core.quit()

